package org.ah.libgdx.demo3dproject;

import com.badlogic.gdx.ApplicationAdapter;
import com.badlogic.gdx.Gdx;
import com.badlogic.gdx.assets.AssetManager;
import com.badlogic.gdx.graphics.GL20;
import com.badlogic.gdx.graphics.PerspectiveCamera;
import com.badlogic.gdx.graphics.g3d.ModelBatch;
import com.badlogic.gdx.graphics.g3d.particles.ParticleEffect;
import com.badlogic.gdx.graphics.g3d.particles.ParticleEffectLoader;
import com.badlogic.gdx.graphics.g3d.particles.ParticleSystem;
import com.badlogic.gdx.graphics.g3d.particles.batches.BillboardParticleBatch;
import com.badlogic.gdx.graphics.g3d.particles.batches.ModelInstanceParticleBatch;
import com.badlogic.gdx.graphics.g3d.particles.emitters.Emitter;
import com.badlogic.gdx.graphics.g3d.particles.emitters.RegularEmitter;
import com.badlogic.gdx.math.Matrix4;
import com.badlogic.gdx.math.Vector3;

public class Demo3DProject extends ApplicationAdapter {

    private static final String FLAME_PFX = "flame.pfx";

    private boolean loadingAssets;

    private AssetManager assetManager;

    private PerspectiveCamera camera;

    private ModelBatch modelBatch;

    private ParticleSystem particleSystem;
    private ParticleEffect effect;
    private ModelInstanceParticleBatch modelInstanceParticleBatch;
    private BillboardParticleBatch billboardParticleBatch;

    @Override
    public void create() {
        int width = Gdx.graphics.getWidth();
        int height = Gdx.graphics.getHeight();

        assetManager = new AssetManager();

        modelBatch = new ModelBatch();

        camera = new PerspectiveCamera(67, width, height);
        camera.near = 0.1f;
        camera.far = 500;
        camera.viewportWidth = width;
        camera.viewportHeight = height;
        camera.position.set(new Vector3(10f, 10f, -5f));
        camera.up.set(new Vector3(0f, 1f, 0f));
        camera.lookAt(0f, 0f, 0f);
        camera.update();

        particleSystem = ParticleSystem.get();
        modelInstanceParticleBatch = new ModelInstanceParticleBatch();
        particleSystem.add(modelInstanceParticleBatch);

        billboardParticleBatch = new BillboardParticleBatch();
        particleSystem.add(billboardParticleBatch);

        ParticleEffectLoader.ParticleEffectLoadParameter loadParam = new ParticleEffectLoader.ParticleEffectLoadParameter(particleSystem.getBatches());
        assetManager.load(FLAME_PFX, ParticleEffect.class, loadParam);

        loadingAssets = true;
    }

    @Override
    public void render() {
        if (loadingAssets && assetManager.update()) {
            ParticleEffect originalEffect = assetManager.get(FLAME_PFX);
            effect = originalEffect.copy();
            effect.init();
            particleSystem.add(effect);

            Emitter emitter = effect.getControllers().first().emitter;
            RegularEmitter reg = (RegularEmitter) emitter;
            reg.setEmissionMode(RegularEmitter.EmissionMode.Enabled);

            Matrix4 effectTransform = new Matrix4();
            effectTransform.idt().setToTranslation(2f, 2f, 2f);
            effect.setTransform(effectTransform);

            loadingAssets = false;
        }

        if (!loadingAssets) {

            particleSystem.update();

            Gdx.gl.glClearColor(0f, 0f, 0f, 1f);
            Gdx.gl.glClear(GL20.GL_COLOR_BUFFER_BIT | GL20.GL_DEPTH_BUFFER_BIT);

            modelBatch.begin(camera);

            billboardParticleBatch.setCamera(camera);
            particleSystem.begin();
            particleSystem.draw();
            particleSystem.end();
            modelBatch.render(particleSystem);

            modelBatch.end();
        }
    }

}
